#include <vector>

class CustomHelper : public RActionImpl<CustomHelper> {
   using Result_t = CustomHisto;
   std::vector<std::shared_ptr<CustomHisto>> histograms_;
   std::vector<bool> is_c:onfigured_;
 public:
    CustomHelper() {
      const auto n_slots = ROOT::IsImplicitMTEnabled() ? ROOT::GetImplicitMTPoolSize() : 1; /// Get number of slots from Pool size, however RDataFrame can use less slots
      for (std::size_t i = 0; i < n_slots; ++i) {
        is_initialized_.push_back(false);
        results_.emplace_back(std::make_shared<CustomHisto>());
      }
    }
    
    void InitTask(TTreeReader * reader, unsigned int slot) {
      if (!is_initialized_[slot]) {
        ConfigureHistogram(reader, slot); /// Here I initialize the binnning of my custom histogram.
        is_initialized_[slot] = true; /// binning is only initialized once.
      }
    }
 
    template <typename... Args>
    void Exec(unsigned int slot, Args &&... args) {
      histograms_[slot]->Fill(args);
    }
 
    void Initialize() { /* noop */}
 
    void Finalize() { 
      std::vector<std::shared_ptr<CustomHisto>>
      for (std::size_t slot = 1; slot < results_.size(); ++slot) {
        others.push_back(results_[slot]); /// I can implement a check here if the slot has been initialized to prevent the error when merging. (is_initialized_)
      }
      results_[0]->Merge(&others); /// The helper will try to merge all slots. If on check it will fail here
    }

    std::shared_ptr<Result_t> GetResultPtr() const { return results_[0]; }
 
    std::string GetActionName() { return "CustomHelper"; }
 };
